%% Data assimilation with the Lorenz 1963 model
% Javier Amezcua
% University of Reading, November 2012

% This file contains the controls for (1) generating a nature run, (b)
% generating artificial observations and (3) performing data assimilation
% with the Lorenz 1963 model including parameter estimation

close all
clear all
home

%% 1.a The nature run
% Let us run perform a 'free' run of the model, which we will consider the
% truth 
% The maximum time and the initial conditions
tmax = 40;
x0 = [-10,10,20];
N = length(x0); % the number of variables in the model

% The parameters are: [sigma,beta,rho]
param_t = [10,8/3,28];
%param_t = [0,0,0];

Nparam = length(param_t);

% computing the nature run
[t,xt] = lorenz63(x0,tmax,param_t);

%% 1.b Plot
% Let us visualize the behavior of the model
fsize = 14;
figure
 for i=1:3
  subplot(3,1,i)
   plot(t,xt(:,i),'k','LineWidth',2); hold on
   ylabel(['x^{(',num2str(i),')}'],'FontSize',fsize)
   xlabel('time','FontSize',fsize)
   set(gca,'FontSize',fsize)
   grid on
 end
 
figure
 plot3(xt(:,1),xt(:,2),xt(:,3),'k','LineWidth',2); hold on 
 xlabel('x^{(1)}','FontSize',fsize)
 ylabel('x^{(2)}','FontSize',fsize)
 zlabel('x^{(3)}','FontSize',fsize)
 title(['\sigma=',num2str(param_t(1,1)),...
        ' \beta=',num2str(param_t(1,2)),...
        ' \rho=',num2str(param_t(1,3))],'FontSize',fsize)
 
 xlim([-25,25])
 ylim([-25,25])
 zlim([0,65])
 set(gca,'FontSize',fsize)
 grid on

%% 2.a The observations
% select an observation operator
%% Option 1: Observe all 
L = 3;
H = eye(L,L);

%% Option 2: Observe only x and z
L = 2;
H = [1,0,0;0,0,1];

%%
% frequency of observations
freqobs = 12;
% an array with the variables that we are observing 
observed_vars = H*(1:1:N)';
% error variance of observations (independent error assumed)
var_obs = 2; 
% generating the observations
[tobs,y,R] = gen_obs(t,xt,freqobs,H,var_obs);

%% 2.b. Plot to check model and obs 
fsize = 14;
figure
 for i=1:3
  subplot(3,1,i)
   plot(t,xt(:,i),'k','LineWidth',2); hold on
   pos_aux = find(observed_vars==i);
   if ~isempty(pos_aux)
    scatter(tobs,y(:,pos_aux),20,'r','filled'); hold on
   end
   ylabel(['x^{(',num2str(i),')}'],'FontSize',fsize)
   xlabel('time','FontSize',fsize)
   set(gca,'FontSize',fsize)
   grid on
 end
 
%% 3.a Data Assimilation and parameter estimation using ensemble methods
% The assimilation method, choose 'SEnKF' or 'ETKF'
rho = 0.;

met = 'ETKF';
M = 20;

param0 = [9,3,30];
%param0 = [6,7,14];

alpha = 0.25;

[Xb,xb,Xa,xa,Parama,parama] = kfs_lor63_pe(x0,param0,t,tobs,y,H,R,rho,alpha,M,met);

%% 3.b. Plot to check model, obs and assim 
fsize = 14;
figure
 for i=1:3
 subplot(3,1,i)
  plot(t,xt(:,i),'k','LineWidth',2); hold on
  plot(t,squeeze(Xb(:,i,:)),'--b'); hold on
  plot(t,squeeze(Xa(:,i,:)),'--m'); hold on
  pos_aux = find(observed_vars==i);
  if ~isempty(pos_aux)
   scatter(tobs,y(:,pos_aux),20,'r','filled'); hold on
  end
  ylabel(['x^{(',num2str(i),')}'],'FontSize',fsize)
  xlabel('time','FontSize',fsize)
  set(gca,'FontSize',fsize)
  grid on
 end

paramt_time = ones(length(tobs),1)*param_t;
fsize = 14;
figure
 for i=1:Nparam
 subplot(Nparam,1,i)
  plot(tobs,paramt_time(:,i),'k','LineWidth',2); hold on
  plot(tobs,squeeze(Parama(:,i,:)),'--m'); hold on
  ylabel(['parameter^{(',num2str(i),')}'],'FontSize',fsize)
  xlabel('time','FontSize',fsize)
  set(gca,'FontSize',fsize)
  grid on
 end
 
 

%% 4.a Check performance computing rmse and spread
% the analysis window has the same length as the observation frequency (in this case)
anawin = freqobs;
% Compute the spread and rmse for background and analysis
[rmseb,spreadb] = rmse_spread(xt,xb,Xb,anawin);
[rmsea,spreada] = rmse_spread(xt,xa,Xa,anawin);

%% 4.b
fsize = 14;
figure
 subplot(2,2,[1,2])
  plot(tobs,rmseb,'b','LineWidth',2); hold on
  plot(tobs,rmsea,'m','LineWidth',2); hold on
  legend('bgd','ana')
  title('root mean squared error','FontSize',fsize)
  xlabel('time','FontSize',fsize)
  set(gca,'FontSize',fsize)
  grid on

 subplot(2,2,3)
  plot(tobs,rmseb,'b','LineWidth',2); hold on
  plot(tobs,spreadb,'--k','LineWidth',2); hold on
  legend('rmse','spread')
  title('background','FontSize',fsize)
  xlabel('time','FontSize',fsize)
  set(gca,'FontSize',fsize)
  grid on
 
 subplot(2,2,4)
  plot(tobs,rmsea,'m','LineWidth',2); hold on
  plot(tobs,spreada,'--k','LineWidth',2); hold on
  legend('rmse','spread')
  title('analysis','FontSize',fsize)
  xlabel('time','FontSize',fsize)
  set(gca,'FontSize',fsize)
  grid on
 

















 
 