function [Xb,x_b,Xa,x_a] = kfs_lor63(x0_t,t,tobs,y,H,R,rho,M,met)
 % [Xb,x_b,Xa,x_a] = kfs_lor63(x0_t,t,tobs,y,H,R,rho,M,met)
 
 % Data assimilation for Lorenz 1963 using Ensemble Kalman Filters
 
 % inputs    - x0_t, the real initial position [1,N]
 %           - t, time array of the model
 %           - tobs, time array of the observations
 %           - y, the observations
 %           - H, observation matrix
 %           - R, the observational error covariance matrix 
 %           - rho, inflation for P. notice we multiply (1+rho)*Xpert or P*(1+rho)^2
 %           - M, the ensemble size
 %           - met, a string containing the method: 'SEnKF', 'ETKF'
 
 % outputs   - Xb, the background ensemble 3D array [time,vars,members]
 %           - xb, background mean
 %           - Xa, the background ensemble 3D array [time,vars,members]
 %           - xa, the analysis array

%% General settings
 % number of variables
 N = length(x0_t);
 % for the true time
 tstep_truth = t(2)-t(1);
 % and for the analysis (we assimilate everytime we get observations)
 tstep_obs = tobs(2)-tobs(1);
 % the ratio
 o2t = tstep_obs/tstep_truth;
 % precreate the arrays for background and analysis
 Xb = NaN(length(t),N,M);
 Xa = NaN(length(t),N,M);

%% For the original background ensemble
 % Two options: fixed and random
 %back0 = 'fixed';
 back0 = 'random';
 
 desv = R(1,1);
 
 % fixed initial conditions for our ensemble, (created ad hoc)
 if strcmp(back0,'fixed') 
  xaux = zeros(N,M); 
  for j=1:N;   xaux(j,:) = linspace(x0_t(1,j)-sqrt(desv),x0_t(1,j)+sqrt(desv),M);  end
  for jm=1:M;  for jn=1:N;     Xb(1,jn,jm) = xaux(jn,jm);      end;        end
 end

 % random initial conditions for our ensemble, let's perturb the real x0
 % using errors with the magnitudes of R
 if strcmp(back0,'random') 
  for jm=1:M;  for jn=1:N;     Xb(1,jn,jm) = x0_t(1,jn) + sqrt(desv)*randn(1);     end;    end
 end

 clear desv
 
 % since we don't have obs at t=0 the first analysis is the same as background
 Xa(1,:,:) = Xb(1,:,:);

%% the following cycle contains evolution and assimilation
 % for all the time steps
 for j=1:length(tobs)-1
  % first we evolve the ensemble members
  % evolve from analysis!
  xold = Xa((j-1)*o2t+1,:,:); % [1,n,m]       
  xold = squeeze(xold); % [n,m]
  % time goes forward
  xnew = evolvemembers(xold,tstep_truth,o2t); % needs [n,m] arrays, gives [o2t+1,n,m]!
  % the new background
  Xb((j-1)*o2t+2:j*o2t+1,:,:) = xnew(2:o2t+1,:,:); % [o2t,n,m]
  Xa((j-1)*o2t+2:j*o2t+1,:,:) = xnew(2:o2t+1,:,:); % [o2t,n,m]
  % the assimilation
  Xa_aux = enkfs(Xb(j*o2t+1,:,:),y(j+1,:),H,R,rho,met); % gets a [n,m] matrix
  Xa_aux = shiftdim(Xa_aux,-1); % [1,n,m]
  Xa(j*o2t+1,:,:) = Xa_aux; % introduce the auxiliary variable in the array
  %j
 end
 % the background and analysis mean
 x_b = mean(Xb,3); % [t2,n,m] -> [t2,n]
 x_a = mean(Xa,3); % [t1,n,m] -> [t1,n]
end

%% Evolving the members
function xnew = evolvemembers(xold,tstep_truth,o2t)
 % xnew = evolvemembers(xold,tstep_truth,o2t)
 % inputs:  - xold, a [n,m] matrix of initial conditions (in every step) for the m members and n variables
 %          - tstep_truth, the time step used in the nature run
 %          - o2t, frequency of observtions in time steps
 % outputs: - xnew, a [o2t+1,n,m] array with the analysis
 t_anal = o2t*tstep_truth;
 tam = size(xold); %[n,m]
 xnew = NaN(o2t+1,tam(1),tam(2)); %[o2t+1,n,m]
 
 for j=1:tam(2)
  [taux,xaux] = lorenz63(xold(:,j)',t_anal,[]); %gives a vector with size [o2t+1,n]
  xnew(:,:,j) = xaux;
 end
end

%% The EnKF algorithms
function Xa = enkfs(Xb,y,H,R,rho,met)
 % Xa = enkfs(Xb,y,H,R,rho,met)
 % performs the analysis using different EnKF methods
 % inputs: - Xb, the ensemble background [1,N,M]
 %         - y, the observations [1,L]
 %         - H, the observation matrix [L,N]
 %         - R, the obs error covariance [L,L]
 %         - rho, inflation for P. notice we multiply (1+rho)*Xpert or P*(1+rho)^2
 %         - met, a string that indicated what method to use
 % Output: - Xa, the full analysis ensemble [N,M]
 
 % General settings
 % the background information
 Xb = squeeze(Xb); %[1,N,M]->[N,M]
 y = y'; % [1,L] -> [L,1]
 
 % number of state variables, observations and ensemble members
 [N,M] = size(Xb); 
 [L,N] = size(H);
 
 % Auxiliary matrices that will ease the computation of averages and
 % covariances
 U = ones(M,M)/M;   I = eye(M,M);
 
 % The ensemble is inflated (rho can be zero)
 Xb_pert = (1+rho)*Xb*(I-U);
 Xb = Xb_pert + 1/M*Xb*ones(M,1)*ones(1,M);

% Now, we choose from one of three methods 
%% Stochastic Ensemble Kalman Filter 
 if strcmp(met,'SEnKF')
  % create the ensemble in Y-space
  Yb = NaN(L,M);
  
  % Map every ensemble member into observation space
  for jm=1:M;     Yb(:,jm) = H*Xb(:,jm);    end
    
  % The matrix of perturbations
  Xb_pert = Xb*(I-U);
  Yb_pert = Yb*(I-U);
  
  % The Kalman gain matrix
  Khat = 1/(M-1)* Xb_pert*Yb_pert' * inv(1/(M-1)*Yb_pert*Yb_pert'+R);
  
  % Fill Xa (the analysis matrix) member by member using perturbed observations
  Xa = NaN(N,M);
  for jm = 1:M
   yaux = y + sqrtm(R)*randn(size(y));
   Xa(:,jm) = Xb(:,jm) + Khat*(yaux-Yb(:,jm));
  end
 end

%% Ensemble Transform Kalman Filter
 if strcmp(met,'ETKF') % if we choose the transform
  % create the ensemble in Y-space
  Yb = NaN(L,M);
  
  for jm=1:M;     Yb(:,jm) = H*Xb(:,jm);    end
  % means and perts 
  xb_bar = Xb*1/M*ones(M,1);
  Xb_pert = Xb*(I-U);
  yb_bar = Yb*1/M*ones(M,1);
  Yb_pert = Yb*(I-U);   
  
  % the method
  Pa_ens = inv((M-1)*eye(M,M)+Yb_pert'*inv(R)*Yb_pert); 
  Wa = sqrtm((M-1)*Pa_ens); % matrix square root (symmetric)
  wa = Pa_ens*Yb_pert'*inv(R)*(y-yb_bar);
  Xa_pert = Xb_pert*Wa;   
  xa_bar = xb_bar + Xb_pert*wa;
  Xa = Xa_pert + xa_bar*ones(1,M);
 end
end





